--------------------------------------------------------
--
-- This is the dcm module which generates all clock signals used inside the DCC.
--
--
---------------------------------------------------------

library ieee,mylib;
use ieee.std_logic_1164.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;
entity clocks is 
    port (
	   		reset			: in std_logic;
	   		n_strobe		: out std_logic;
	   		mclk			: out std_logic;
	   		mclkn			: out std_logic;
	   		mclk270		: out std_logic;
	   		mclk90		: out std_logic;
           	ttc_clk		: out std_logic ;
           	lock			: out std_logic_vector(3 downto 0);
				clk_pads		: inout clk_pad 
           );

end clocks;

architecture clock_arch of clocks is 
attribute LOC : string;
attribute LOC of ttcbufg : label is "BUFGMUX5S";
attribute LOC of mclkbufg : label is "BUFGMUX7P";
attribute LOC of mclknbufg : label is "BUFGMUX6S";
attribute LOC of memclkbufg : label is "BUFGMUX2S";
attribute LOC of mclk270bufg : label is "BUFGMUX4S";
attribute LOC of mclk90bufg : label is "BUFGMUX5P";
signal pciclk: std_logic := '0';
signal mclk_dcm: std_logic := '0';
signal mclkn_dcm: std_logic := '0';
signal memclk_dcm: std_logic := '0';
signal memclk : std_logic := '0';
signal mclk_int : std_logic := '0';
signal strobe: std_logic := '0';
signal pci_clk: std_logic := '0';
signal pci_clk_dcm: std_logic := '0';
signal mclk270_dcm: std_logic := '0';
signal mclk270_i: std_logic := '0';
signal mclk90_dcm: std_logic := '0';
signal mclk90_i: std_logic := '0';
signal mclk_lock_dl : std_logic := '0';
signal pciclk2x_lock_dl : std_logic := '0';
signal rst_mclk : std_logic := '0';
signal lock_i: std_logic_vector(3 downto 0) := (others => '0');
begin
lock <= lock_i;
lock_i(3 downto 2) <= "11";
lock0_dl: delayN
  generic map(N => 17) port map( d => lock_i(0), ce => '1', c => memclk, q => pciclk2x_lock_dl );
rst_mclk <= not pciclk2x_lock_dl;
process(mclk_int)
begin
  if(mclk_int'event and mclk_int = '1')then
    n_strobe <= strobe;
    strobe <= not strobe;
  end if;
end process;
-- mclk generation
pciclkbuf: ibufg port map ( o => pciclk, i => clk_pads.clk_in);
memclkbufg : BUFG port map (O=>memclk, I=>memclk_dcm);
DCMmemclk: dcm
	generic map(CLK_FEEDBACK => "2X",
				   STARTUP_WAIT => TRUE)  
	port map	
         ( CLKIN   => pciclk ,
           CLKFB   => memclk ,
           RST     => reset ,
           DSSEN   => '0' ,
           PSINCDEC=> '0' ,
           PSEN    => '0' ,
           PSCLK   => '0' ,
           CLK2X   => memclk_dcm ,
           LOCKED  => lock_i(0));
-- mclk descew
mclk <= mclk_int;
mclk90 <= mclk90_i;
mclk270 <= mclk270_i;
mclkbufg : BUFG port map (O=>mclk_int, I=>mclk_dcm);
mclk90bufg : BUFG port map (O=>mclk90_i, I=>mclk90_dcm);
mclknbufg : BUFG port map (O=>mclkn, I=>mclkn_dcm);
mclk270bufg : BUFG port map (O=>mclk270_i, I=>mclk270_dcm);
DCMmclk: dcm 
	generic map(
				   STARTUP_WAIT => TRUE)  
	port map	
         ( CLKIN   => memclk ,
           CLKFB   => mclk_int ,
           RST     => rst_mclk ,
           DSSEN   => '0' ,
           PSINCDEC=> '0' ,
           PSEN    => '0' ,
           PSCLK   => '0' ,
           CLK0    => mclk_dcm ,
			  clk90    => mclk90_dcm,
           CLK180  => mclkn_dcm ,
           CLK270  => mclk270_dcm ,
           LOCKED  => lock_i(1));
i_memclk_ext: ddr_io
    port map(
          clk => mclk270_i,
          clkn => mclk90_i,
          iclk => '0',
          iclkn => '0',
          pad => clk_pads.ram_clk,
			 t => '0',
          d0 => '1',
          d1 => '0');
i_memclk_extn: ddr_io
    port map(
          clk => mclk270_i,
          clkn => mclk90_i,
          iclk => '0',
          iclkn => '0',
          pad => clk_pads.ram_clk_n,
			 t => '0',
          d0 => '0',
          d1 => '1');
-- this ensures pci clock about 2ns earlier than pci data
process(mclk270_i)
begin
  if(mclk270_i'event and mclk270_i = '1')then
    clk_pads.pci1_clk <= strobe;
    clk_pads.pci2_clk <= strobe;
    clk_pads.pci3_clk <= strobe;
  end if;
end process;
ttcbufg : BUFG port map (O=>ttc_clk, I=> clk_pads.ttc_clk);
end clock_arch;

