--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 


library IEEE;
use IEEE.STD_LOGIC_1164.all;

package dcc_pack is
subtype vector16 is std_logic_vector(15 downto 0);
type array_x8y16 is array(7 downto 0) of vector16;
type array_x4y16 is array(3 downto 0) of vector16;
subtype vector3 is std_logic_vector(2 downto 0);
type array_x8y3 is array(7 downto 0) of vector3;
subtype vector4 is std_logic_vector(3 downto 0);
type array_x4y4 is array(3 downto 0) of vector4;
type lvds_pad is record
	sync: 			std_logic;
	busy: 			std_logic;
	ready: 			std_logic;
	ovfl: 			std_logic;
end record;
type LED_pad is record
	c: 				std_logic;
	i: 				std_logic;
	o:		 			std_logic;
end record;
type lrb_pad is record
	data: 			std_logic_vector(15 downto 0);
	ctrl: 			std_logic_vector(7 downto 0);
end record;
type pci3_pad is record
	data: 			std_logic_vector(15 downto 0);
	ctrl: 			std_logic_vector(11 downto 0);
end record;
type ram_pad is record
	rasn:				std_logic;
	casn:				std_logic;
	wen:				std_logic;
	clke:				std_logic;
	dm:				std_logic;
	dqs:				std_logic;
	csn:				std_logic;
	data: 			std_logic_vector(31 downto 0);
	addr: 			std_logic_vector(12 downto 0);
end record;
type ttc_pad is record
	sda:				std_logic;
	scl:				std_logic;
	reset_b:			std_logic;
	ttcready:		std_logic;
	l1accept:		std_logic;
--	bcntstr:			std_logic;
	bcntres:			std_logic;
	evcntres:			std_logic;
--	evcnthstr:		std_logic;
--	evcntlstr:		std_logic;
	dberrstr:		std_logic;
	sinerrstr:		std_logic;
	brcststr1:		std_logic;
	brcst: 			std_logic_vector(7 downto 2);
	id:	 			std_logic_vector(13 downto 0);
--	bcnt: 			std_logic_vector(11 downto 0);
end record;
type clk_pad is record
	clk_in:			std_logic;
	ttc_clk:			std_logic;
	pci1_clk:		std_logic;
	pci2_clk:		std_logic;
	pci3_clk:		std_logic;
	ram_clk:			std_logic;
	ram_clk_n:		std_logic;
end record;
type slink_pad is record
	clock:			std_logic;
	lffn:				std_logic;
	ldownn:			std_logic;
	urstn:			std_logic;
	utestn:			std_logic;
	uwen:				std_logic;
	uctrln:			std_logic;
	usf: 				std_logic_vector(3 downto 0);
	udw: 				std_logic_vector(1 downto 0);
	ud:				std_logic_vector(63 downto 0);
end record;
-----------------------------------------------------------------------
COMPONENT ttc_if
	PORT(
		ttc_clk : IN std_logic;
		clk : IN std_logic;
		reset : IN std_logic;
		run : IN std_logic;
		n_strobe : IN std_logic;
		rst_ttc : IN std_logic;
		wr_ttc_id : IN std_logic;
		rd_ttc_id : IN std_logic;
		wr_i2c : IN std_logic;
		rd_i2c : IN std_logic;
		w_data : IN std_logic_vector(31 downto 0);
		addr : IN std_logic_vector(13 downto 0);
		en_brcst : IN std_logic;
		evn_fifo_full : IN std_logic;    
		ttc_pads : INOUT ttc_pad;      
		r_data : OUT std_logic_vector(31 downto 0);
		ttc_start : OUT std_logic;
		ttc_stop : OUT std_logic;
		ttc_soft_reset : OUT std_logic;
		ttc_LRB_stat : OUT std_logic;
		ttc_ready : OUT std_logic;
		ttc_serr : OUT std_logic;
		ttc_derr : OUT std_logic;
		ttc_bcnt_err : OUT std_logic;
		sync_lost : OUT std_logic;
		inc_oc : OUT std_logic;
		inc_l1ac : OUT std_logic;
		inc_bcnterr : OUT std_logic;
		inc_serr : OUT std_logic;
		inc_derr : OUT std_logic;
      state : in std_logic_vector(3 downto 0);
		event_number_avl : OUT std_logic;
		event_number : OUT std_logic_vector(71 downto 0)
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT clocks
	PORT(
		reset : IN std_logic;    
		clk_pads : INOUT clk_pad;      
		n_strobe : OUT std_logic;
		mclk : OUT std_logic;
		mclkn : OUT std_logic;
		mclk90 : OUT std_logic;
		mclk270 : OUT std_logic;
		ttc_clk : OUT std_logic;
		lock : OUT std_logic_vector(3 downto 0)
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT refresh
	PORT(
		clk : IN std_logic;
		not_locked : IN std_logic;
		mode_set : IN std_logic;          
		mrs : OUT std_logic;
		start_init : OUT std_logic;
		ddr_csn : OUT std_logic;
		ddr_clke : OUT std_logic;
		ref : OUT std_logic
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT lrb_if
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		ttc_reset : IN std_logic;
		n_strobe : IN std_logic;
		run : IN std_logic;
		htr_buf_full : IN std_logic_vector(8 downto 0);
		out_gnt : IN std_logic;
		auto_monitor : IN std_logic;
		monitor_start : IN std_logic;
		rd_lrb_mon : IN std_logic;
		addr : IN std_logic_vector(13 downto 0);    
		lrb_pads : INOUT lrb_pad;      
		dav : OUT std_logic;
		out_rqst : OUT std_logic;
		lrb_out : OUT std_logic_vector(35 downto 0);
		lrb_mon_ready : OUT std_logic;
		lrb_en : OUT std_logic_vector(8 downto 0);
		r_data : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT slink
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		n_strobe : IN std_logic;
		rd_slink_pntr : IN std_logic;
		rd_slink_data : IN std_logic;
		wr_slink_data : IN std_logic;
		addr : IN std_logic_vector(13 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		en_slink : IN std_logic;
		slink64 : IN std_logic;
		slink_ctrl : IN std_logic;
		slink_din : IN std_logic_vector(35 downto 0);
		slink_we : IN std_logic;
		slink_test : IN std_logic;    
		slink_pads : INOUT slink_pad;      
		r_data : OUT std_logic_vector(31 downto 0);
		inc_slink_wc : OUT std_logic;
		inc_slink_bc : OUT std_logic;
		slink_send : OUT std_logic;
		slink_full : OUT std_logic;
		slink_down : OUT std_logic
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT sdram_ctl
	PORT(
		reset : IN std_logic;
      mem_test: in STD_LOGIC;
      mem_done: in STD_LOGIC;
      mem_fail : out std_logic;
		chk_done : out std_logic;
      rd_debug: IN STD_LOGIC;
		clk : IN std_logic;
		clkn : IN std_logic;
		clk90 : IN std_logic;
		clk270 : IN std_logic;
		not_locked : IN std_logic;
		addr : IN std_logic_vector(13 downto 0);
		n_strobe : IN std_logic;
		flush_cache : IN std_logic;
		rd_ddr_data : IN std_logic;
		cache_chk : IN std_logic;
		ddr_pa : IN std_logic_vector(6 downto 0);
		ddr_wfifo_we : IN std_logic;
		ddr_w_req : IN std_logic;
		ddr_w_length : IN std_logic_vector(5 downto 0);
		ddr_w_addr : IN std_logic_vector(20 downto 0);
		ddr_w_data : IN std_logic_vector(31 downto 0);
		htr_buf_req : IN std_logic;
		htr_buf_addr : IN std_logic_vector(14 downto 0);
		htr_buf_length : IN std_logic_vector(5 downto 0);
		htr_buf_re : IN std_logic;    
		ram_pads : INOUT ram_pad;      
		r_data : OUT std_logic_vector(31 downto 0);
		cache_hit : OUT std_logic;
		ddr_w_gnt : OUT std_logic;
		htr_buf_gnt : OUT std_logic;
		htr_buf_data : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT ddr_w_queue
	PORT(
		reset : IN std_logic;
		mem_test : IN std_logic;
		clk : IN std_logic;
		w_addr : IN std_logic_vector(11 downto 0);
		event_avl : IN std_logic;
		lrb1_avl : IN std_logic;
		lrb2_avl : IN std_logic;
		event_we : IN std_logic;
		lrb1_we : IN std_logic;
		lrb2_we : IN std_logic;
		lrb_data : IN std_logic_vector(33 downto 0);
		event_data : IN std_logic_vector(33 downto 0);
		ddr_w_gnt : IN std_logic;
		rd_debug : IN std_logic;
		w_data : IN std_logic_vector(31 downto 0);
		r_data : OUT std_logic_vector(31 downto 0);
		addr : IN std_logic_vector(13 downto 0);
		run : IN std_logic;
		is_high : IN std_logic;
		n_strobe : IN std_logic;
		pci3_cycle : IN std_logic;
		wr_ddr_data : IN std_logic;
		ddr_pa : IN std_logic_vector(6 downto 0);          
		event_re : OUT std_logic;
		lrb1_re : OUT std_logic;
		lrb2_re : OUT std_logic;
		src_sel : OUT std_logic_vector(1 downto 0);
		ddr_w_dav : OUT std_logic;
		ddr_w_req : OUT std_logic;
		ddr_w_length : OUT std_logic_vector(5 downto 0);
		ddr_w_addr : OUT std_logic_vector(20 downto 0);
		ddr_w_data : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT lrb_info
	PORT(
		reset : IN std_logic;
		clk : IN std_logic;
		strobe : IN std_logic;
		rd_status_cntr : IN std_logic;
		addr : IN std_logic_vector(13 downto 0);
		wr_stateLUT : IN std_logic;
		rd_stateLUT : IN std_logic;
		w_data : IN std_logic_vector(31 downto 0);
		src_sel : IN std_logic;
		lrb1_we : IN std_logic;
		lrb2_we : IN std_logic;
		lrb_data : IN std_logic_vector(35 downto 0);
		lrb1_data : IN std_logic_vector(35 downto 0);
		lrb2_data : IN std_logic_vector(35 downto 0);
		lrb_wa : IN std_logic_vector(10 downto 0);
		evn_ra : IN std_logic_vector(10 downto 0);
		smry_ra : IN std_logic_vector(10 downto 0);          
		r_data : OUT std_logic_vector(31 downto 0);
		HTR_ow : OUT std_logic;
		HTR_bz : OUT std_logic;
		HTR_state : OUT std_logic_vector(1 downto 0);
		lrb_evn : OUT std_logic_vector(40 downto 0);
		summary : OUT std_logic_vector(31 downto 0);
		lrb_total_wc : OUT std_logic_vector(9 downto 0);
		wr_lrb_total_wc : OUT std_logic
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT monitor
	PORT(
		reset : IN std_logic;
		clk : IN std_logic;
		w_data : IN std_logic_vector(31 downto 0);
		n_strobe : IN std_logic;
		wr_scale : IN std_logic;
		rd_scale : IN std_logic;
		inc_ddr_pa : IN std_logic;
		ddr_pa : IN std_logic_vector(6 downto 0);
		rd_mon_wc : IN std_logic;
		mon_re : IN std_logic;
		slink_we : IN std_logic;
		slink_data : IN std_logic_vector(35 downto 0);          
		r_data : OUT std_logic_vector(31 downto 0);
		mon_buf_empty : OUT std_logic;
		mon_buf_full : OUT std_logic;
		mon_buf_afull : OUT std_logic;
		mon_avl : OUT std_logic;
		mon_we : OUT std_logic;
		inc_mon_cntr : OUT std_logic;
		mon_data : OUT std_logic_vector(33 downto 0);
		mon_wp : OUT std_logic_vector(6 downto 0)
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT evt_bldr
	PORT(
		reset : IN std_logic;
		reset_err : IN std_logic;
		clk : IN std_logic;
		version : IN std_logic_vector(15 downto 0);
		event_number_avl : IN std_logic;
		event_number : IN std_logic_vector(71 downto 0);
		lrb_en : IN std_logic_vector(17 downto 0);
		src_sel : IN std_logic_vector(1 downto 0);
		lrb1_we : IN std_logic;
		lrb2_we : IN std_logic;
		lrb1_data : IN std_logic_vector(35 downto 0);
		lrb2_data : IN std_logic_vector(35 downto 0);
		mon_wp : IN std_logic_vector(6 downto 0);
		addr : IN std_logic_vector(13 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		run : IN std_logic;
		n_strobe : IN std_logic;
		is_high : IN std_logic;
		rd_evn_pntr : IN std_logic;
		rd_evn_buf : IN std_logic;
		wr_timeout : IN std_logic;
		rd_timeout : IN std_logic;
		wr_src_id : IN std_logic;
		rd_src_id : IN std_logic;
		rd_status_cntr : IN std_logic;
		wr_stateLUT : IN std_logic;
		rd_stateLUT : IN std_logic;
		wr_mmLUT : IN std_logic;
		rd_mmLUT : IN std_logic;
		rd_counters : IN std_logic;
		inc_l1ac : IN std_logic;
		inc_bcnterr : IN std_logic;
		inc_oc : IN std_logic;
		inc_serr : IN std_logic;
		inc_derr : IN std_logic;
		inc_mon_cntr : IN std_logic;
		inc_slink_wc : IN std_logic;
		inc_slink_bc : IN std_logic;
		htr_buf_gnt : IN std_logic;
		htr_buf_data : IN std_logic_vector(31 downto 0);
		TTS : IN std_logic_vector(3 downto 0);
		slink_full : IN std_logic;
		rd_debug : IN std_logic;          
		ovfl_warning : OUT std_logic;
		evn_fifo_full : OUT std_logic;
		lrb_data : OUT std_logic_vector(33 downto 0);
		w_addr : OUT std_logic_vector(11 downto 0);
		r_data : OUT std_logic_vector(31 downto 0);
		HTR_ow : OUT std_logic;
		HTR_bz : OUT std_logic;
		err_state : OUT std_logic_vector(1 downto 0);
		htr_status : OUT std_logic_vector(14 downto 0);
		htr_error : OUT std_logic_vector(14 downto 0);
		htr_buf_req : OUT std_logic;
		htr_buf_addr : OUT std_logic_vector(14 downto 0);
		htr_buf_length : OUT std_logic_vector(5 downto 0);
		htr_buf_re : OUT std_logic;
		htr_buf_full : OUT std_logic_vector(14 downto 0);
		slink_we : OUT std_logic;
		slink_data : OUT std_logic_vector(35 downto 0)
		);
END COMPONENT;
-----------------------------------------------------------------------
COMPONENT pci3
	PORT(
		clk : IN std_logic;
		not_locked : IN std_logic;
		run : IN std_logic;
		mem_test : IN std_logic;
		ddr_w_gnt : IN std_logic;
		chk_done : IN std_logic;
		r_data : IN std_logic_vector(31 downto 0);
		n_strobe : IN std_logic;
		ttc_soft_reset : IN std_logic;
		cache_hit : IN std_logic;    
		pci3_pads : INOUT pci3_pad;      
		reset : OUT std_logic;
		reset_err : OUT std_logic;
		mem_done : OUT std_logic;
		addr : OUT std_logic_vector(13 downto 0);
		w_data : OUT std_logic_vector(31 downto 0);
		pci3_cycle : OUT std_logic;
		is_high : OUT std_logic;
		wr_config : OUT std_logic;
		rd_config : OUT std_logic;
		wr_scale : OUT std_logic;
		rd_scale : OUT std_logic;
		wr_timeout : OUT std_logic;
		rd_timeout : OUT std_logic;
		wr_src_id : OUT std_logic;
		rd_src_id : OUT std_logic;
		rd_mon_wc : OUT std_logic;
		rst_ttc : OUT std_logic;
		wr_ttc_id : OUT std_logic;
		rd_ttc_id : OUT std_logic;
		wr_i2c : OUT std_logic;
		rd_i2c : OUT std_logic;
		rd_evn_pntr : OUT std_logic;
		rd_evn_buf : OUT std_logic;
		rd_counters : OUT std_logic;
		rd_status_cntr : OUT std_logic;
		wr_stateLUT : OUT std_logic;
		rd_stateLUT : OUT std_logic;
		wr_mmLUT : OUT std_logic;
		rd_mmLUT : OUT std_logic;
		rd_lrb_en : OUT std_logic;
		monitor_start : OUT std_logic;
		rd_lrb_mon_pci1 : OUT std_logic;
		rd_lrb_mon_pci2 : OUT std_logic;
		slink_rst : OUT std_logic;
		wr_slink_data : OUT std_logic;
		rd_slink_data : OUT std_logic;
		rd_slink_pntr : OUT std_logic;
		wr_ddr_data : OUT std_logic;
		rd_ddr_data : OUT std_logic;
		cache_chk : OUT std_logic;
		flush_cache : OUT std_logic;
		rd_ddr_pa : OUT std_logic;
		inc_ddr_pa : OUT std_logic;
		ld_ddr_pa : OUT std_logic;
		rd_status : OUT std_logic;
		rd_debug : OUT std_logic
		);
END COMPONENT;
COMPONENT rd_lrb_wc
	PORT(
		reset : IN std_logic;
		clk : IN std_logic;
		strobe : IN std_logic;
		lrb_total_wc : IN std_logic_vector(9 downto 0);
		wp_addr : IN std_logic_vector(3 downto 0);
		wr_lrb_total_wc : IN std_logic;
		rd_cntr_que : IN std_logic;          
		scan_cntr_que_a : OUT std_logic;
		cntr_que : OUT std_logic_vector(16 downto 0);
		empty_cntr_que : OUT std_logic
		);
END COMPONENT;
COMPONENT crc16d32
	PORT(
		clk : IN std_logic;
		init_crc : IN std_logic;
		we_crc : IN std_logic;
		d : IN std_logic_vector(31 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT led_driver
	PORT(
		clk : IN std_logic;
		strobe : IN std_logic;
		err_in : IN std_logic_vector(23 downto 0);
		leds : IN std_logic_vector(23 downto 0);    
		led_pads : INOUT led_pad;      
		display_err : OUT std_logic
		);
END COMPONENT;
COMPONENT rnd_test
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		strobe : IN std_logic;
		mem_test : IN std_logic;
		ddr_w_gnt : IN std_logic;
		chk_done : IN std_logic;          
		mem_done : OUT std_logic;
		rd_blk : OUT std_logic;
		wr_blk : OUT std_logic;
		inc_pa : OUT std_logic
		);
END COMPONENT;
end dcc_pack;
package body dcc_pack is
end dcc_pack;
