library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity monitor is
    port (
-- reset input, should be connected to the inverted LOCK output of dll
        reset: in STD_LOGIC;
-- system clock input
        clk: in STD_LOGIC;
-- download data write
        w_data: in STD_LOGIC_VECTOR (31 downto 0);
        r_data: out STD_LOGIC_VECTOR (31 downto 0);
        n_strobe: in STD_LOGIC;
-- monitor event scaling factor
        wr_scale: in STD_LOGIC;
        rd_scale: in STD_LOGIC;
-- monitor event word count
        mon_buf_empty: out STD_LOGIC;
        mon_buf_full: out STD_LOGIC;
        mon_buf_afull: out STD_LOGIC;
        inc_ddr_pa: in STD_LOGIC;
        rd_mon_wc: in STD_LOGIC;
-- monitored event data
        mon_re: in STD_LOGIC;
        mon_avl: out STD_LOGIC;
        mon_we: out STD_LOGIC;
        mon_data: out STD_LOGIC_VECTOR (33 downto 0);
        mon_wp: out STD_LOGIC_VECTOR (6 downto 0);
        ddr_pa: in STD_LOGIC_VECTOR (6 downto 0);
        inc_mon_cntr: out STD_LOGIC;
-- data in write enable
        slink_we: in STD_LOGIC;
-- header and trailer are required to be marked as control data
        slink_data: in STD_LOGIC_VECTOR (35 downto 0)
    );
end monitor;

architecture my_arch of monitor is
signal strobe: std_logic := '0';
signal scale: std_logic_vector(31 downto 0) := x"00000000";
signal not_scale: std_logic_vector(15 downto 0) := (others => '0');
signal mon_wc: std_logic_vector(15 downto 0) := (others => '0');
signal ce_wc_reg_wa: std_logic := '0';
signal ec_fifo_ra: std_logic := '0';
signal mon_we_i: std_logic := '0';
signal mon_re_dl: std_logic := '0';
signal down_count: std_logic := '0';
signal evt_cnt: std_logic_vector(7 downto 0) := (others => '0');
signal fifo_wa: std_logic_vector(8 downto 0) := (others => '0');
signal fifo_ra: std_logic_vector(8 downto 0) := (others => '0');
signal wc_reg_wa: std_logic_vector(9 downto 0) := (others => '0');
signal wc_reg_ra: std_logic_vector(9 downto 0) := (others => '0');
signal wc_reg: std_logic_vector(31 downto 0) := (others => '0');
signal fifo_din: std_logic_vector(35 downto 0) := (others => '0');
signal fifo_dout: std_logic_vector(35 downto 0) := (others => '0');
signal ce_scale: std_logic := '0';
signal ld_scale: std_logic := '0';
signal up_count: std_logic := '0';
signal mon_enp: std_logic := '0';
signal mon_en: std_logic := '0';
signal scale_carry: std_logic := '0';
signal pending: std_logic := '0';
signal mon_buf_empty_i: std_logic := '0';
-------------
begin
mon_buf_empty <= mon_buf_empty_i;
mon_buf_full <= evt_cnt(7);
mon_buf_afull <= evt_cnt(6);
mon_we <= mon_we_i;
mon_data <= fifo_dout(33 downto 0);
mon_wp <= wc_reg_wa(6 downto 0);
wc_reg_ra(6 downto 0) <= ddr_pa;
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
  end if;
end process;
-- monitor event fifo
fifo_din <= slink_data;
mon_fifo: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => fifo_din(31 downto 0),
  			  DIPA => fifo_din(35 downto 32),
  			  DIB => gnd32,
  			  DIPB => gnd4,
  			  ENA => mon_en,
  			  ENB => '1',
  			  WEA => slink_we,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => fifo_wa,
			  ADDRB => fifo_ra,
			  DOB => fifo_dout(31 downto 0),
			  DOPB => fifo_dout(35 downto 32)
			  );
---------------------------------		
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   fifo_wa <= (others => '0');
	   fifo_ra <= (others => '0');
	 else
	   if(slink_we = '1' and mon_en = '1')then
		  if(slink_data(32) = '1')then
		    fifo_wa(8 downto 6) <= fifo_wa(8 downto 6) + 1;
		    fifo_wa(5 downto 0) <= (others => '0');
		  else
		    fifo_wa <= fifo_wa + 1;
		  end if;
		end if; 
	   if(ec_fifo_ra = '1')then
		  if(fifo_dout(32) = '1')then
		    fifo_ra(8 downto 6) <= fifo_ra(8 downto 6) + 1;
		    fifo_ra(5 downto 0) <= (others => '0');
		  else
		    fifo_ra <= fifo_ra + 1;
		  end if;
		end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   mon_avl <= '0';
      ec_fifo_ra <= '0';
	   mon_re_dl <= '0';
	   mon_we_i <= '0';
    else
      if(ec_fifo_ra = '1' or fifo_wa(8 downto 6) = fifo_ra(8 downto 6))then
        mon_avl <= '0';
		else
        mon_avl <= '1';
      end if;
      if(mon_re = '1')then
        ec_fifo_ra <= '1';
      elsif(fifo_ra(5 downto 0) = "111111" or fifo_dout(32) = '1')then
        ec_fifo_ra <= '0';
      end if;
	   mon_re_dl <= mon_re;
	   mon_we_i <= mon_re_dl or (mon_we_i and not fifo_dout(32) and ec_fifo_ra);
    end if;
  end if;
end process;
--------------------------------------------------------------
i_mon_wc: l_cntr
  generic map(N => 13, ld_plus_one => true)
  port map(
    d => gnd16(12 downto 0),
-- load to one when header is seen
    load => fifo_dout(33),
    reset => '0',
    set => '0',
    ec => mon_we_i,
    en_cnt => '1',
    clk => clk,
	 q => mon_wc(12 downto 0));
mon_wc(15 downto 13) <= "000";
i_wc_reg: RAMB16_S18_S18
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => mon_wc,
  			  DIPA => gnd2,
  			  DIB => gnd16,
  			  DIPB => gnd2,
  			  ENA => '1',
  			  ENB => strobe,
  			  WEA => ce_wc_reg_wa,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => mon_buf_empty_i,
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => wc_reg_wa,
			  ADDRB => wc_reg_ra,
			  DOB => wc_reg(15 downto 0));
wc_reg(31 downto 16) <= x"0000";
wc_reg_wa(9 downto 8)	<= "00";
wc_reg_ra(9 downto 8)	<= "00";
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   wc_reg_wa(7 downto 0) <= "00000000";
		wc_reg_ra(7) <= '0';
	 else
	   if(ce_wc_reg_wa = '1')then
		  wc_reg_wa(7 downto 0) <= wc_reg_wa(7 downto 0) + 1;
		end if;
		if(strobe = '1' and inc_ddr_pa = '1' and ddr_pa = "1111111")then
		  wc_reg_ra(7) <= not wc_reg_ra(7);
		end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(wc_reg_wa(6 downto 0) = wc_reg_ra(6 downto 0))then
	   mon_buf_empty_i <= wc_reg_wa(7) xnor wc_reg_ra(7);
    else
	   mon_buf_empty_i <= '0';
	 end if;
  end if;
end process;
i_evt_cnt: updown_cntr
  generic map(N => 8)
  port map(
    rst => reset,
    set => '0',
    ce => '1',
    up => up_count,
    down => down_count,
    clk => clk,
	 q => evt_cnt);
down_count <= '1' when strobe = '1' and evt_cnt /= x"00" and inc_ddr_pa = '1' else '0';
r_data <= wc_reg when rd_mon_wc = '1' else (others => 'Z');
------------------------------------------------------------------------
---- monitor event scaling
------------------------------------------------------------------------
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
		if(wr_scale = '1')then
        scale(15 downto 0) <= w_data(15 downto 0);
		end if;
	 end if;
  end if;
end process;
scale(31 downto 16) <= x"0000";
r_data <= scale when rd_scale = '1' else (others => 'Z');
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   ce_scale <= '0';
	   ld_scale <= '0';
	   up_count <= '0';
	   ce_wc_reg_wa <= '0';
	   inc_mon_cntr <= '0';
	 else
	   ce_scale <= slink_we and slink_data(33);
	   ld_scale <= (mon_en or pending) and slink_we and slink_data(33);
	   up_count <= mon_en and slink_we and slink_data(33);
	   ce_wc_reg_wa <= mon_we_i and fifo_dout(32);
		if(strobe = '1')then
	     inc_mon_cntr <= ce_wc_reg_wa or (mon_we_i and fifo_dout(32));
		end if;
	 end if;
  end if;
end process;
not_scale <= not scale(15 downto 0);
i_mon_en: l_cntr
  generic map(N => 16)
  port map(
    d => not_scale,
    load => ld_scale,
    reset => '0',
    set => '0',
    ec => ce_scale,
    en_cnt => '1',
    clk => clk,
	 carry => scale_carry);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
-- After reset, the first event will always be recorded
      mon_enp <= '1';
	   pending <= '0';
    elsif(slink_we = '1' and slink_data(32) = '1')then
		mon_enp <= not evt_cnt(7) and (scale_carry or pending);
		pending <= evt_cnt(7) and (scale_carry or pending);
	 end if;
  end if;
end process;
mon_en <= mon_enp or slink_data(35);
end my_arch;
