library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;

entity pci3 is
    port (
        clk: in STD_LOGIC;
-- reset, active high
        reset: out STD_LOGIC;
        reset_err: out STD_LOGIC;
		  not_locked : IN std_logic;
		  run : IN std_logic;
		  mem_test : IN std_logic;
		  ddr_w_gnt : IN std_logic;
		  chk_done : IN std_logic;
		  mem_done : OUT std_logic;
        addr: out STD_LOGIC_VECTOR (13 downto 0);
        w_data: out STD_LOGIC_VECTOR (31 downto 0);
        r_data: in STD_LOGIC_VECTOR (31 downto 0);
        pci3_cycle: out STD_LOGIC;
        is_high: out STD_LOGIC;
        n_strobe: in STD_LOGIC;
-- configure register
        wr_config: out STD_LOGIC;
        rd_config: out STD_LOGIC;
-- monitor event scaling factor
        wr_scale: out STD_LOGIC;
        rd_scale: out STD_LOGIC;
-- synchronization timeout setting
        wr_timeout: out STD_LOGIC;
        rd_timeout: out STD_LOGIC;
-- source ID
        wr_src_id: out STD_LOGIC;
        rd_src_id: out STD_LOGIC;
-- monitor event word count
        rd_mon_wc: out STD_LOGIC;
-- TTCrx access
	 	  ttc_soft_reset : in std_logic;
        rst_ttc: out STD_LOGIC;
        wr_ttc_id: out STD_LOGIC;
        rd_ttc_id: out STD_LOGIC;
        wr_i2c: out STD_LOGIC;
        rd_i2c: out STD_LOGIC;
        rd_evn_pntr: out STD_LOGIC;
        rd_evn_buf: out STD_LOGIC;
-- counter read out
        rd_counters: out STD_LOGIC;
-- HTR status counter read out
        rd_status_cntr: out STD_LOGIC;
        wr_stateLUT: out STD_LOGIC;
        rd_stateLUT: out STD_LOGIC;
        wr_mmLUT: out STD_LOGIC;
        rd_mmLUT: out STD_LOGIC;
-- LRB enable
        rd_lrb_en: out STD_LOGIC;
-- LRB monitor data
        monitor_start: out STD_LOGIC;
        rd_lrb_mon_pci1: out STD_LOGIC;
        rd_lrb_mon_pci2: out STD_LOGIC;
-- slink access
        slink_rst: out STD_LOGIC;
        wr_slink_data: out STD_LOGIC;
        rd_slink_data: out STD_LOGIC;
        rd_slink_pntr: out STD_LOGIC;
-- sdram access
        wr_ddr_data: out STD_LOGIC;
        rd_ddr_data: out STD_LOGIC;
		  cache_chk: out STD_LOGIC;
        cache_hit: in STD_LOGIC;
        flush_cache: out STD_LOGIC;
        rd_ddr_pa: out STD_LOGIC;
        inc_ddr_pa: out STD_LOGIC;
        ld_ddr_pa: out STD_LOGIC;
-- read status
        rd_status: out STD_LOGIC;
-- pci3 pads
		  pci3_pads: inout pci3_pad;
-- debugging control
		  rd_debug: out std_logic
    );
end pci3;

architecture pci3_arch of pci3 is
signal ld_pa: std_logic := '0';
signal inc_pa: std_logic := '0';
signal mem_test_dl: std_logic := '0';
signal pa: std_logic_vector(31 downto 0);
signal dcc_ctrl: std_logic_vector(11 downto 0) := (others => '0');
signal dcc_data_i: std_logic_vector(15 downto 0) := (others => '0');
signal dcc_data_o: std_logic_vector(31 downto 0) := (others => '0');
signal w_data_i: std_logic_vector(31 downto 0) := (others => '0');
signal dcc_data_oq: std_logic_vector(15 downto 0) := (others => '0');
signal tqq: std_logic_vector(15 downto 0) := (others => '0');
signal ar: std_logic_vector(14 downto 0) := (others => '0');
signal strobe: std_logic := '0';
signal reset_i: std_logic := '0';
signal ld_ar: std_logic := '0';
signal cache_chk_i: std_logic := '0';
signal wr_ddr_data_i: std_logic := '0';
signal retry: std_logic := '0';
signal ec_ar: std_logic := '0';
signal last_word: std_logic := '0';
signal en_read: std_logic := '0';
signal en_read_dl: std_logic_vector(2 downto 0) := (others => '0');
signal is_read: std_logic := '0';
signal burst: std_logic := '0';
signal pci_cycle: std_logic := '0';
signal tq: std_logic := '0';
signal cmd: std_logic := '0';
signal high: std_logic := '0';
signal was_high: std_logic := '0';
-- random memory test signals
signal rd_blk: std_logic := '0'; 
signal wr_blk: std_logic := '0'; 
signal inc_pa_m: std_logic := '0'; 
begin
is_high <= high;
reset <= reset_i;
addr <= ar(13 downto 0);
ld_ddr_pa <= ld_pa;
inc_ddr_pa <= inc_pa;
w_data <= w_data_i;
wr_ddr_data <= wr_ddr_data_i;
i_ar: l_cntr
  generic map(N=> 14)
  port map(
    d => dcc_data_i(15 downto 2),
    load => ld_ar,
    reset => '0',
    set => '0',
    ec => strobe,
    en_cnt => ec_ar,
    clk => clk,
    q => ar(13 downto 0)
  );
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
    if(strobe = '1')then
      pci3_cycle <= pci_cycle;
	   ec_ar <= high and not ld_ar;
      ld_ar <= dcc_ctrl(1) and not dcc_ctrl(0) and not ld_ar;
      high <= (dcc_ctrl(1) or en_read) and not high;
		was_high <= high;
      w_data_i(31 downto 16) <= w_data_i(15 downto 0);
      w_data_i(15 downto 0) <= dcc_data_i;
		if(was_high = '1')then
        dcc_data_o <= r_data;
		else
        dcc_data_o <= dcc_data_o(15 downto 0) & r_data(15 downto 0);
		end if;
      if(ld_ar = '1')then
-- convert monitor event space to memory space
        ar(14) <= (not run and w_data_i(0)) or (not w_data_i(0) and dcc_data_i(15));
      end if;
	 end if;
  end if;
end process;
-- 
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(strobe = '1')then
      if(pci_cycle = '0')then
        en_read_dl <= "000";
	   else
        en_read_dl <= en_read_dl(1 downto 0) & en_read;
		end if;  
	 end if;
  end if;
end process;
tq <= not en_read_dl(2);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset_i = '1')then
      cache_chk <= '0';
	   retry <= '0';
	 elsif(strobe = '1')then
      cache_chk <= (not mem_test and cache_chk_i) or (mem_test and rd_blk);
	   if(cache_chk_i = '1')then
	     retry <= not cache_hit;
		end if;
	 end if;
  end if;
end process;
pci_cycle <= dcc_ctrl(2); -- lt_tsr(0) = '1'
process(clk)
begin
  if(clk'event and clk = '1')then
    if(mem_test = '1')then
	   flush_cache <= not mem_test_dl;
	 else
	   flush_cache <= strobe and (ld_pa or inc_pa or wr_ddr_data_i or(cache_chk_i and not cache_hit and not retry));
	 end if;
	 mem_test_dl <= mem_test;
    if(strobe = '1')then
-- universe2 burst never crosses 32 word boundary
	   if(en_read = '1' and (ar(4 downto 1) = "1111" and (ar(0) = '1' or ec_ar = '1')))then
		  last_word <= '1';
		else
		  last_word <= '0';
		end if;
      burst <= (burst and pci_cycle) or (ld_ar and w_data_i(14));
      cache_chk_i <=  ld_ar and not w_data_i(15) and w_data_i(13);
	 end if;
    if(pci_cycle = '0')then
	   en_read <= '0';
	   is_read <= '0';
    elsif(strobe = '1')then
      en_read <= (en_read and not high) or (is_read and not last_word);
      is_read <= (ld_ar and not w_data_i(15) and not w_data_i(13)) or (cache_chk_i and cache_hit) or (is_read and burst and not last_word);
	 end if;
  end if;
end process;
--	commands
process(clk,not_locked)
begin
  if(not_locked = '1')then
    reset_i <= '1';
    slink_rst <= '1';
    rst_ttc <= '0';
    reset_err <= '0';
    monitor_start <= '0';
  elsif(clk'event and clk = '1')then
    if(strobe = '1')then
-- when high = '1', dcc_data_i has the lower word
 	   reset_i <= (cmd and high and dcc_data_i(0)) or ttc_soft_reset;
      slink_rst <= cmd and high and dcc_data_i(1);
      rst_ttc <= cmd and high and dcc_data_i(2);
      reset_err <= (cmd and high and (dcc_data_i(3) or dcc_data_i(0))) or ttc_soft_reset;
      monitor_start <= cmd and high and dcc_data_i(4);
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
	   if(mem_test = '1')then
		  wr_ddr_data_i <= wr_blk;
		  rd_ddr_data <= '0';
      elsif(ar(14) = '1')then
        wr_ddr_data_i <= dcc_ctrl(0) and dcc_ctrl(1) and not run;
        rd_ddr_data <= en_read and high;
      else
        wr_ddr_data_i <= '0';
        rd_ddr_data <= '0';
      end if; 
      if(ar(14 downto 12) = "001")then
        rd_evn_buf <= en_read and high and not run;
      else
        rd_evn_buf <= '0';
      end if; 
      if(ar(14 downto 9) = "000110")then
        wr_slink_data <= dcc_ctrl(0) and dcc_ctrl(1) and high and not run;
        rd_slink_data <= en_read and high;
      else
        wr_slink_data <= '0';
        rd_slink_data <= '0';
      end if; 
      if(ar(14 downto 7) = "00000001")then
        rd_counters <= en_read and high;
      else
        rd_counters <= '0';
      end if; 
      if(ar(14 downto 8) = "0000010")then
        rd_status_cntr <= en_read and high;
      else
        rd_status_cntr <= '0';
      end if; 
      if(ar(14 downto 7) = x"06")then
        wr_stateLUT <= dcc_ctrl(0) and dcc_ctrl(1);
        rd_stateLUT <= en_read and high;
      else
        wr_stateLUT <= '0';
        rd_stateLUT <= '0';
      end if; 
      if(ar(14 downto 7) = x"07")then
        wr_mmLUT <= dcc_ctrl(0) and dcc_ctrl(1);
        rd_mmLUT <= en_read and high;
      else
        wr_mmLUT <= '0';
        rd_mmLUT <= '0';
      end if; 
      if(ar = "000000000000000")then
        cmd <= dcc_ctrl(0) and dcc_ctrl(1);
        rd_status <= en_read and high;
      else
        cmd <= '0';
        rd_status <= '0';
      end if; 
      if(ar = "000000000000001")then
        wr_config <= dcc_ctrl(0) and dcc_ctrl(1);
        rd_config <= en_read and high;
      else
        wr_config <= '0';
        rd_config <= '0';
      end if; 
      if(ar = "000000000000010")then
        wr_scale <= dcc_ctrl(0) and dcc_ctrl(1) and not run and high;
        rd_scale <= en_read and high;
      else
        wr_scale <= '0';
        rd_scale <= '0';
      end if; 
      if(ar = "000000000000011")then
        rd_lrb_en <= en_read and high;
      else
        rd_lrb_en <= '0';
      end if; 
      if(ar = "000000000000100")then
        wr_ttc_id <= dcc_ctrl(0) and dcc_ctrl(1) and not run and high;
        rd_ttc_id <= en_read and high;
      else
        wr_ttc_id <= '0';
        rd_ttc_id <= '0';
      end if; 
      if(ar = "000000000000101")then
        wr_i2c <= dcc_ctrl(0) and dcc_ctrl(1) and high;
        rd_i2c <= en_read and high;
      else
        wr_i2c <= '0';
        rd_i2c <= '0';
      end if; 
      if(ar = "000000000000110")then
        wr_timeout <= dcc_ctrl(0) and dcc_ctrl(1) and not run and high;
        rd_timeout <= en_read and high;
      else
        wr_timeout <= '0';
        rd_timeout <= '0';
      end if; 
      if(ar = "000000000000111")then
        wr_src_id <= dcc_ctrl(0) and dcc_ctrl(1) and not run and high;
        rd_src_id <= en_read and high;
      else
        wr_src_id <= '0';
        rd_src_id <= '0';
      end if;
		if(mem_test = '1')then 
        ld_pa <= '0';
        inc_pa <= inc_pa_m;
      elsif(ar = "000000000100000")then
        ld_pa <= dcc_ctrl(0) and dcc_ctrl(1) and not run and high;
        inc_pa <= dcc_ctrl(0) and dcc_ctrl(1) and run and high;
      else
        ld_pa <= '0';
        inc_pa <= '0';
      end if; 
      if(ar = "000000000100000")then
        rd_ddr_pa <= en_read and high;
      else
        rd_ddr_pa <= '0';
      end if; 
      if(ar = "000000000100001")then
        rd_mon_wc <= en_read and high;
      else
        rd_mon_wc <= '0';
      end if; 
      if(ar = "000000000100010")then
        rd_evn_pntr <= en_read and high;
      else
        rd_evn_pntr <= '0';
      end if; 
      if(ar = "000000000100011")then
        rd_slink_pntr <= en_read and high;
      else
        rd_slink_pntr <= '0';
      end if; 
      if(ar(14 downto 3) = "000000000101")then
        rd_debug <= en_read and high;
      else
        rd_debug <= '0';
      end if; 
      if(ar(14 downto 5) = "0000001000")then
        rd_lrb_mon_pci1 <= en_read and high;
      else
        rd_lrb_mon_pci1 <= '0';
      end if;
      if(ar(14 downto 5) = "0000001001")then
        rd_lrb_mon_pci2 <= en_read and high;
      else
        rd_lrb_mon_pci2 <= '0';
      end if;
	 end if;
----- output signals
  end if;
end process;
dcc_ctrl(11 downto 8) <= "1111";
dcc_ctrl(6) <= tq;
-- pci3 pins
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
-- if cache miss toggle dcc_ctrl(7) to signal retry
      dcc_ctrl(7) <= dcc_ctrl(7) xor (cache_chk_i and not cache_hit);
	   pci3_pads.ctrl(11 downto 6) <= dcc_ctrl(11 downto 6);
	   dcc_ctrl(2 downto 0) <= pci3_pads.ctrl(2 downto 0);
	   dcc_data_i <= pci3_pads.data;
	   dcc_data_oq <= dcc_data_o(31 downto 16);
		for i in tqq'range loop
		  tqq(i) <= tq;
		end loop;
	 end if;
  end if;
end process;
g_dcc_data: for i in dcc_data_oq'range generate
  dcc_data: pci3_pads.data(i) <= dcc_data_oq(i) when tqq(i) = '0' else 'Z';
end generate;
-- random memory test
i_rnd_test: rnd_test PORT MAP(
		clk => clk,
		reset => reset_i,
		strobe => strobe,
		mem_test => mem_test,
		ddr_w_gnt => ddr_w_gnt,
		mem_done => mem_done,
		chk_done => chk_done,
		rd_blk => rd_blk,
		wr_blk => wr_blk,
		inc_pa => inc_pa_m
	);
end pci3_arch;
