library IEEE,mylib;
use IEEE.std_logic_1164.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity slink is
    port (
-- main clock 128MHz
        clk: in STD_LOGIC;
-- resets slink, active high
        reset: in STD_LOGIC;
-- slink monitoring
        n_strobe: in STD_LOGIC;
        rd_slink_pntr: in STD_LOGIC;
        rd_slink_data: in STD_LOGIC;
        wr_slink_data: in STD_LOGIC;
        addr: in STD_LOGIC_VECTOR (13 downto 0);
        w_data: in STD_LOGIC_VECTOR (31 downto 0);
        r_data: out STD_LOGIC_VECTOR (31 downto 0);
-- data in from dcc logic
        en_slink: in STD_LOGIC;
        slink64: in STD_LOGIC;
-- header and trailer are required to be marked as control data
        slink_ctrl: in STD_LOGIC;
        slink_din: in STD_LOGIC_VECTOR (35 downto 0);
-- data in write enable
        slink_we: in STD_LOGIC;
--	signal for LED display
        slink_send: out STD_LOGIC;
-- fifo full
        slink_full: out STD_LOGIC;
-- link down output
        slink_down: out STD_LOGIC;
-- slink word count output
        inc_slink_wc: out STD_LOGIC;
        inc_slink_bc: out STD_LOGIC;
-- slink test in
        slink_test: in STD_LOGIC;
		  slink_pads: inout slink_pad);
end slink;

architecture slink_arch of slink is
signal strobe: std_logic := '0';
signal uclk: std_logic := '0';
signal was_data: std_logic := '0';
signal ec_slink_pin: std_logic := '0';
signal slink_re: std_logic := '0';
signal uwe: std_logic := '0';
signal urstn: std_logic := '0';
signal urstn_dl: std_logic := '0';
signal buf_empty: std_logic := '0';
signal buf_we: std_logic := '0';
signal slink_up: std_logic := '0';
signal slink_down_i: std_logic := '0';
signal ud: std_logic_vector(63 downto 0) := (others => '0');
signal buf_wa: std_logic_vector(8 downto 0) := (others => '0');
signal buf_rap: std_logic_vector(8 downto 0) := (others => '0');
signal buf_ra: std_logic_vector(8 downto 0) := (others => '0');
signal word_cnt: std_logic_vector(7 downto 0) := (others => '0');
signal buf_din: std_logic_vector(31 downto 0) := (others => '0');
signal buf_dout: std_logic_vector(31 downto 0) := (others => '0');
signal buf_cin: std_logic_vector(3 downto 0) := (others => '0');
signal buf_cout: std_logic_vector(3 downto 0) := (others => '0');
signal pntr: std_logic_vector(31 downto 0) := (others => '0');
signal rst_cycle: std_logic_vector(9 downto 0) := (others => '0');
begin
slink_send <= uwe;
slink_down <= slink_down_i;
-- after changing udw, must reset slink!!!
slink_pads.udw <= "11" when slink64 = '1' else "00";
slink_pads.urstn <= urstn;
slink_pads.usf <= "0000";
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
	 if(reset = '1')then
	   was_data <= '0';
	 elsif(strobe = '1' and uwe = '1' and uclk = '1')then
	   was_data <= not buf_cout(0);
	 end if;
    if(strobe = '1')then
	   inc_slink_wc <= uwe and uclk;
-- control word after data is end of event
	   inc_slink_bc <= uwe and uclk and was_data and buf_cout(0);
      uclk <= not uclk;
	 end if;
    slink_pads.clock <= uclk;
	 ec_slink_pin <= uclk and n_strobe;
	 if(ec_slink_pin = '1')then
      slink_pads.ud <= ud;
      uwe <= slink_re;
      slink_pads.uwen <= not uwe;
      slink_pads.utestn <= not slink_test;
      slink_pads.uctrln <= not buf_cout(0);
	 end if;
  end if;
end process;
-- The low word is always latched to delay two clock cycles
ud(63 downto 32) <= buf_dout;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
	   ud(31 downto 0) <= buf_dout;
	 end if;
  end if;
end process;
--	   urstn <= slink_pads.ldownn and en_slink;
i_rst_cycle:l_cntr
  generic map(N=> 10)
  port map(
    d => gnd16(9 downto 0),
    load => '0',
    reset => slink_up,
    set => '0',
    ec => ec_slink_pin,
    en_cnt => '1',
    clk => clk,
    q => rst_cycle );
slink_up <= not slink_down_i or not en_slink or slink_test;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   slink_down_i <= '1';
		urstn <= '0';
    elsif(strobe = '1')then
	   slink_down_i <= not slink_pads.ldownn;
-- It is not mentioned in the slink spec what state urstn should be in test mode
--	   urstn <= not slink_down_i and (urstn or not urstn_dl) and en_slink;
	   urstn <= not rst_cycle(9) and (urstn or not urstn_dl);
	 end if;
  end if;
end process;
i_urstn_dl: delayN
  generic map(N => 9) port map( d => urstn, ce => strobe, c => clk, q => urstn_dl );
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   buf_empty <= '1';
	   slink_full <= '0';
	   slink_re <= '0';
	 elsif(strobe = '1')then
	   if(word_cnt(7 downto 1) = "0000000")then
	     buf_empty <= not word_cnt(0) or slink_re;
		else
	     buf_empty <= '0';
		end if;
-- leave at least 128 words for 64 words htr_burst
	   slink_full <= word_cnt(7) and word_cnt(6);
		if(uclk = '0')then
		  slink_re <=  slink_pads.lffn and urstn and urstn_dl and not buf_empty;
		else
		  slink_re <= slink_re and slink64;
		end if;  
	 end if;
  end if;
end process;
-- when not in run mode, data can be written to	SLINK from pci3 interface
process(clk)
begin
  if(clk'event and clk = '1')then
    if(wr_slink_data = '1')then
	   buf_din <= w_data;
	 else
	   buf_din <= slink_din(31 downto 0);
	 end if;
	 buf_cin(0) <= slink_ctrl;
	 buf_we <= ((slink_we and not slink_din(35)) or (wr_slink_data and strobe)) and en_slink;
  end if;
end process;
buf_cin(3 downto 1) <= "000";
buf_rap <= buf_ra when en_slink = '1' else addr(8 downto 0);
-- slink data buffer. When SLINK is not enabled, the last 512 words 
-- can be read back through pci3 interface for debugging
buf: ramb16_s36_s36
	  generic map(SIM_COLLISION_CHECK => "NONE")
     port map(
         dia => gnd32, 
         dipa => gnd4, 
         addra => buf_rap,
         clka => clk, 
         ssra => '0', 
         ena => strobe, 
         wea => '0',
			doa => buf_dout,
			dopa => buf_cout,
         dib => buf_din,
         dipb => buf_cin,
         addrb => buf_wa,
         clkb => clk,
         ssrb => '0',
         enb => '1',
         web => buf_we);
-- slink buffer write pointer
i_buf_wa:l_cntr
  generic map(N=> 9)
  port map(
    d => gnd16(8 downto 0),
    load => '0',
    reset => reset,
    set => '0',
    ec => buf_we,
    en_cnt => '1',
    clk => clk,
    q => buf_wa );
-- slink buffer read pointer. When SLINK is not enabled, read address
-- is determined by pci3 interface
i_buf_ra:l_cntr
  generic map(N=> 9)
  port map(
    d => gnd16(8 downto 0),
    load => '0',
    reset => reset,
    set => '0',
    ec => strobe,
    en_cnt => slink_re,
    clk => clk,
    q => buf_ra );
-- Only send pairs of 32-bit words, even if data width are 32-bits
i_word_cnt: adsuN
   generic map( N => 8)
   port map(
         a => buf_wa(8 downto 1),
         b => buf_ra(8 downto 1),
         ci => '1',
         add => '0',
         s => word_cnt);
r_data <= buf_dout when rd_slink_data = '1' else (others => 'Z');
pntr <= "0000000" & buf_wa & "0000000" & buf_ra;
r_data <= pntr when rd_slink_pntr = '1' else (others => 'Z');
end slink_arch;
